unit AnimatedButton;

interface

uses
   Windows, Messages, Classes, Graphics, Controls;

type
   TAnimatedButtonThread = class(TThread)
   public
      iSpeed: integer;
      FNotify: TNotifyEvent;

      constructor Create(DoNotify: TNotifyEvent);
      procedure Execute; override;
      procedure DoEvent;
   end;

   TAnimatedButton = class(TCustomControl)
   private
      FImageList: TImageList;
      FBitmap: TBitmap;
      FSpeed: integer;
      FSelected: boolean;
      FOnMouseEnter: TNotifyEvent;
      FOnMouseExit: TNotifyEvent;
      FBitmapIndex: integer;
      FThread: TAnimatedButtonThread;
   public
      constructor Create(aOwner: TComponent); override;
      destructor Destroy; override;
   protected
      procedure SetBitmap(bNew: TBitmap);
      procedure SetSelected(bNew: boolean);
      procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
      procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
      procedure Paint; override;
      procedure DoChange(Sender: TObject);
   published
      property Bitmap: TBitmap read FBitmap write SetBitmap;
      property ImageList: TImageList read FImageList write FImageList;
      property Selected: boolean read FSelected write SetSelected;
      property Speed: integer read FSpeed write FSpeed;
      property OnClick;
      property OnMouseEnter: TNotifyEvent read FOnMouseEnter write FOnMouseEnter;
      property OnMouseExit: TNotifyEvent read FOnMouseExit write FOnMouseExit;
   end;

procedure Register;

implementation

{------------------------------------------------------------------------------}

procedure Register;
begin
   RegisterComponents('Samples', [TAnimatedButton]);
end;

{##############################################################################}

constructor TAnimatedButtonThread.Create(DoNotify: TNotifyEvent);
begin
   inherited Create(true);

   FreeOnTerminate := true;
   FNotify := DoNotify;
   iSpeed := 100;
end;

{------------------------------------------------------------------------------}

procedure TAnimatedButtonThread.Execute;
begin
   while not Terminated do
   begin
      SleepEx(iSpeed, false);
      Synchronize(DoEvent);
   end;
end;

{------------------------------------------------------------------------------}

procedure TAnimatedButtonThread.DoEvent;
begin
   FNotify(Self);
end;

{##############################################################################}

constructor TAnimatedButton.Create(aOwner: TComponent);
begin
   inherited Create(aOwner);
   Speed := 10;
   FBitmap := TBitmap.Create;
   ControlStyle := ControlStyle + [csSetCaption, csCaptureMouse];
   FSelected := false;
   Height := 32;
   Width := 32;
   FThread := TAnimatedButtonThread.Create(DoChange);
end;

{------------------------------------------------------------------------------}

destructor TAnimatedButton.Destroy;
begin
   FThread.Terminate;
   FBitmap.Free;

   inherited;
end;

{------------------------------------------------------------------------------}

procedure TAnimatedButton.SetBitmap(bNew: TBitmap);
begin
   FBitmap.Assign(bNew);
   Paint;
end;

{------------------------------------------------------------------------------}

procedure TAnimatedButton.SetSelected(bNew: boolean);
begin
   if bNew = FSelected then
   begin
      Paint;
      exit;
   end;

   FSelected := bNew;
   FThread.iSpeed := FSpeed;

   if bNew = true then begin FBitmapIndex := 0; FThread.Resume; end
   else FThread.Suspend;

   Paint;
end;

{------------------------------------------------------------------------------}

procedure TAnimatedButton.CMMouseLeave(var Message: TMessage);
begin
   if assigned(FOnMouseExit) then FOnMouseExit(Self);
   Paint;
end;

{------------------------------------------------------------------------------}

procedure TAnimatedButton.CMMouseEnter(var Message: TMessage);
begin
   if assigned(FOnMouseEnter) then FOnMouseEnter(Self);
   Paint;
end;

{------------------------------------------------------------------------------}

procedure TAnimatedButton.Paint;
begin
   if (FSelected = true) and (assigned(FImageList)) then FImageList.Draw(Canvas, 0, 0, FBitmapIndex)
   else Canvas.CopyRect(ClientRect, FBitmap.Canvas, ClientRect);
end;

{------------------------------------------------------------------------------}

procedure TAnimatedButton.DoChange(Sender: TObject);
begin
   if assigned(FImageList) then
   begin
      inc(FBitmapIndex);
      if FBitmapIndex = FImageList.Count then FBitmapIndex := 0;
   end;
   Paint;
end;

end.
